package rankine;
import java.awt.geom.Point2D;
import java.util.ArrayList;
import rankine.steamtable.SaturatedVapor;
import rankine.steamtable.SteamTable;
import rankine.steamtable.StateVariable;
import rankine.steamtable.TableData;
import resourcemanager.WordMap;

//--------------------------------------------------------------------//
/**
 * LTCN
 */
public class RankineCycle{

/** ͍ől */
  public static final double P_MAX=50.0;

/** `捏 */
  public static final int INC=20;

/** C\ */
  protected SteamTable table;

/** ሳ */
  protected double p1;

/**  */
  protected double p2;

/** ^[rCx */
  protected double t3;

/** P-V }̓_i[郊Xg */
  protected ArrayList pv;

/** T-S }̓_i[郊Xg */
  protected ArrayList ts;

/** ^[rd */
  public double wt;

/** |vd */
  public double wp;

/** {C[ŉM */
  public double q1;

/** ŕMM */
  public double q2;

/** M */
  public double efficiency;

/** |v̏ԗ */
  public StateVariable sv1;

/** |vȍԗ */
  public StateVariable sv2;

/** ^[ȑԗ */
  public StateVariable sv3;

/** ^[rȍԗ */
  public StateVariable sv4;

/**
 * LTCN\z
 */
  public RankineCycle(){
    table=new SteamTable();
    p1=0.0;
    p2=0.0;
    t3=0.0;
    pv=new ArrayList();
    ts=new ArrayList();
    wt=0.0;
    wp=0.0;
    q1=0.0;
    q2=0.0;
    efficiency=0.0;
  }

//--------------------------------------------------------------------//
/**
 * C\Ԃ
 * @return C\
 */
  public SteamTable getSteamTable(){
    return table;
  }

//--------------------------------------------------------------------//
/**
 * TCNݒ肷
 * @param p1 ሳ
 * @param p2 
 * @param t3 ^[rCx
 * @throws InputException TCNɖ肪Ƃ
 */
  public void setCondition(double p1,double p2,double t3)
    throws InputException{

    SaturatedVapor sv=table.getSaturatedVapor();
    if(p1>=p2){
      throw new InputException(WordMap.get(WordMap.WARNING1));
    }
    else if(p1<=sv.getTripleP()){
      throw new InputException(WordMap.get(WordMap.WARNING2));
    }
    else if(p1>=sv.getCriticalP()){
      throw new InputException(WordMap.get(WordMap.WARNING3));
    }
    else if(p2>P_MAX){
      throw new InputException(WordMap.get(WordMap.WARNING4));
    }
    else if(t3<=sv.getGasByP(p2).t){
      throw new InputException(WordMap.get(WordMap.WARNING5));
    }
    this.p1=p1;
    this.p2=p2;
    this.t3=t3;
  }

//--------------------------------------------------------------------//
/**
 * }f[^𒊏o
 */
  public void createDiagram(){
    pv.clear();
    ts.clear();
    SaturatedVapor sv=table.getSaturatedVapor();
    TableData cw=table.getCompressedWater();
    TableData hv=table.getSuperHeatedVapor();
    sv1=sv.getLiquidByP(p1);
    addPoint(sv1);
    sv2=cw.getByPS(p2,sv1.s);
    addPoint(sv2);
// QR
    double t21=sv.getCriticalT();
    if(p2<sv.getCriticalP()){
      StateVariable sv21=sv.getLiquidByP(p2);
      double ds=(sv21.s-sv1.s)/(double)INC;
      for(int i=1;i<INC;i++){
      	addPoint(cw.getByPS(p2,sv1.s+ds*(double)i));
      }
      addPoint(sv21);
      StateVariable sv22=sv.getGasByP(p2);
      addPoint(sv22);
      t21=sv22.t;
    }
    else{
      double dt=(t21-sv2.t)/(double)INC;
      for(int i=1;i<=INC;i++){
      	addPoint(cw.getByPT(p2,sv2.t+dt*(double)i));
      }
    }
    double dt=(t3-t21)/(double)INC;
    for(int i=1;i<INC;i++){
      addPoint(hv.getByPT(p2,t21+dt*(double)i));
    }
    sv3=hv.getByPT(p2,t3);
    addPoint(sv3);
// RS
    StateVariable sv31=sv.getGasByS(sv3.s);
    if(sv31.t>=sv1.t){
      double dp=(sv31.p-p2)/(double)INC;
      for(int i=1;i<=INC;i++){
      	addPoint(hv.getByPS(p2+dp*(double)i,sv3.s));
      }
      addPoint(sv31);
      dp=(p1-sv31.p)/(double)INC;
      for(int i=1;i<INC;i++){
      	StateVariable sv32=sv.getGasByP(sv31.p+dp*(double)i);
      	double sr=(sv3.s-sv1.s)/(sv32.s-sv1.s);
      	pv.add(new Point2D.Double(sv1.v+sr*(sv32.v-sv1.v),sv32.p));
      	ts.add(new Point2D.Double(sv3.s,sv32.t));
      }
      sv4=sv.getGasByP(p1);
      double sr=(sv3.s-sv1.s)/(sv4.s-sv1.s);
      sv4.set(sv4.t,p1,sv1.v+sr*(sv4.v-sv1.v),sv1.u+sr*(sv4.u-sv1.u),
      	      sv1.h+sr*(sv4.h-sv1.h),sv3.s);
      addPoint(sv4);
    }
    else{
      double dp=(p1-p2)/(double)INC;
      for(int i=1;i<INC;i++){
      	addPoint(hv.getByPS(p2+dp*(double)i,sv3.s));
      }
      sv4=hv.getByPS(p1,sv3.s);
      addPoint(sv4);
      StateVariable sv41=sv.getGasByP(p1);
      double ds=(sv41.s-sv4.s)/(double)INC;
      for(int i=1;i<INC;i++){
      	addPoint(hv.getByPS(p1,sv4.s+ds*(double)i));
      }
      addPoint(sv41);
    }
    pv.trimToSize();
    ts.trimToSize();
    wp=sv2.h-sv1.h;
    wt=sv3.h-sv4.h;
    q1=sv3.h-sv2.h;
    q2=sv4.h-sv1.h;
    efficiency=(wt-wp)/q1;
  }

//--------------------------------------------------------------------//
/**
 * f[^_ǉ
 * @param sv ԗ
 */
  protected void addPoint(StateVariable sv){
    pv.add(new Point2D.Double(sv.v,sv.p));
    ts.add(new Point2D.Double(sv.s,sv.t));
  }

//--------------------------------------------------------------------//
/**
 * P-V }̖Oatf[^Ԃ
 * @return Oatf[^
 */
  public Point2D[] getLiquidPV(){
    return table.getSaturatedVapor().getLiquidPV();
  }

//--------------------------------------------------------------------//
/**
 * T-S }̖Oatf[^Ԃ
 * @return Oatf[^
 */
  public Point2D[] getLiquidTS(){
    return table.getSaturatedVapor().getLiquidTS();
  }

//--------------------------------------------------------------------//
/**
 * P-V }̖OaCf[^Ԃ
 * @return OaCf[^
 */
  public Point2D[] getGasPV(){
    return table.getSaturatedVapor().getGasPV();
  }

//--------------------------------------------------------------------//
/**
 * T-S }̖OaCf[^Ԃ
 * @return OaCf[^
 */
  public Point2D[] getGasTS(){
    return table.getSaturatedVapor().getGasTS();
  }

//--------------------------------------------------------------------//
/**
 * P-V }̖OatECf[^Ԃ
 * @return Oaf[^
 */
  public Point2D[] getSaturatedPV(){
    Point2D p1[]=table.getSaturatedVapor().getLiquidPV();
    Point2D p2[]=table.getSaturatedVapor().getGasPV();
    Point2D p[]=new Point2D[2*p1.length-1];
    for(int i=0;i<p1.length;i++){
      p[i]=p1[i];
      p[p1.length-1+i]=p2[i];
    }
    return p;
  }

//--------------------------------------------------------------------//
/**
 * T-S }̖OatECf[^Ԃ
 * @return Oaf[^
 */
  public Point2D[] getSaturatedTS(){
    Point2D p1[]=table.getSaturatedVapor().getLiquidTS();
    Point2D p2[]=table.getSaturatedVapor().getGasTS();
    Point2D p[]=new Point2D[2*p1.length-1];
    for(int i=0;i<p1.length;i++){
      p[i]=p1[i];
      p[p1.length-1+i]=p2[i];
    }
    return p;
  }

//--------------------------------------------------------------------//
/**
 * P-V }f[^Ԃ
 * @return P-V }f[^
 */
  public Point2D[] getPV(){
    int size=pv.size();
    Point2D p[]=new Point2D[size];
    for(int i=0;i<size;i++) p[i]=(Point2D)pv.get(i);
    return p;
  }

//--------------------------------------------------------------------//
/**
 * T-S }f[^Ԃ
 * @return T-S }f[^
 */
  public Point2D[] getTS(){
    int size=ts.size();
    Point2D p[]=new Point2D[size];
    for(int i=0;i<size;i++) p[i]=(Point2D)ts.get(i);
    return p;
  }
}
