package rankine;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import gui.BasicViewCanvas;
import rankine.graph.GraphWindow;
import rankine.graph.TextBox;
import resourcemanager.WordMap;

//--------------------------------------------------------------------//
/**
 * LTCN\LoX
 */
public class RankineCycleCanvas extends BasicViewCanvas{

/** LTCN */
  protected RankineCycle cycle;

/** G[bZ[W */
  protected String message;

/** wiF */
  protected Color bgColor;

/** OiF */
  protected Color fgColor;

/** F */
  protected Color subColor;

/** tHg */
  protected Font font;

/** P-V } */
  protected GraphWindow pvWindow;

/** T-S } */
  protected GraphWindow tsWindow;

/** eLXg{bNX */
  protected TextBox infoBox;

/**
 * n}\LoX\z
 * @param cycle LTCN
 * @param message G[bZ[W
 * @param font tHg
 * @param bgColor wiF
 * @param fgColor OiF
 * @param subColor F
 */
  public RankineCycleCanvas(RankineCycle cycle,String message,
      	      	      	    Font font,Color bgColor,Color fgColor,
      	      	      	    Color subColor){
    super();
    this.cycle=cycle;
    this.message=message;
    this.font=font;
    setColors(bgColor,fgColor,subColor);
    pvWindow=new GraphWindow
      ("V["+WordMap.get(WordMap.VOLUME_UNIT)+"]",true,
       "P["+WordMap.get(WordMap.PRESS_UNIT)+"]",false);
    tsWindow=new GraphWindow
      ("S["+WordMap.get(WordMap.ENTOROPY_UNIT)+"]",
       "T["+WordMap.get(WordMap.TEMP_UNIT)+"]");
    infoBox=new TextBox();
  }

//--------------------------------------------------------------------//
/**
 * \Fݒ肷
 * @param bgColor wiF
 * @param fgColor OiF
 * @param subColor F
 */
  protected void setColors(Color bgColor,Color fgColor,Color subColor){
    if(bgColor==null){
      this.bgColor=Color.WHITE;
    }
    else{
      this.bgColor=bgColor;
    }
    if(fgColor==null){
      this.fgColor=Color.BLACK;
    }
    else{
      this.fgColor=fgColor;
    }
    if(subColor==null){
      this.subColor=Color.GRAY;
    }
    else{
      this.subColor=subColor;
    }
    setBackground(this.bgColor);
  }

//--------------------------------------------------------------------//
/**
 * G[bZ[Wݒ肷
 * @param message G[bZ[W
 */
  public void setMessage(String message){
    this.message=message;
  }

//--------------------------------------------------------------------//
/**
 * LTCNԂ
 * @return LTCN
 */
  public RankineCycle getRankineCycle(){
    return cycle;
  }

//--------------------------------------------------------------------//
/**
 * Otf[^ݒ肷
 */
  public void setGraphData(){
    pvWindow.setData(cycle.getSaturatedPV(),cycle.getPV());
    tsWindow.setData(cycle.getSaturatedTS(),cycle.getTS());
    pvWindow.setAxis();
    tsWindow.setAxis();
    infoBox.clear();
    infoBox.addText(WordMap.get(WordMap.TEMP1)
      	      	   +NumberString.toString(cycle.sv1.t)
      	      	   +" ["+WordMap.get(WordMap.TEMP_UNIT)+"]");
    String enUnit=" ["+WordMap.get(WordMap.ENERGY_UNIT)+"]";
    infoBox.addText(WordMap.get(WordMap.Q1)
      	      	   +NumberString.toString(cycle.q1)+enUnit);
    infoBox.addText(WordMap.get(WordMap.Q2)
      	      	   +NumberString.toString(cycle.q2)+enUnit);
    infoBox.addText(WordMap.get(WordMap.WP)
      	      	   +NumberString.toString(cycle.wp)+enUnit);
    infoBox.addText(WordMap.get(WordMap.WT)
      	      	   +NumberString.toString(cycle.wt)+enUnit);
    infoBox.addText(WordMap.get(WordMap.EFFICIENCY)
      	      	   +NumberString.toString(cycle.efficiency));
    infoBox.setSize(bg.getFontMetrics());
  }

//--------------------------------------------------------------------//
/**
 * ĕ`悷
 * @param g `悷OtBbNXReLXg
 */
  public void update(Graphics g){
    if(message!=null){
      drawErrorMessage(bg,message);
    }
    else{
      paintGraph(bg);
    }
    g.drawImage(bgImage,0,0,this);
  }

//--------------------------------------------------------------------//
/**
 * `悷
 * @param g `悷OtBbNXReLXg
 */
  public void paint(Graphics g){
    setPaint();
    if(message!=null){
      drawErrorMessage(bg,message);
    }
    else{
      paintGraph(bg);
    }
    g.drawImage(bgImage,0,0,this);
  }

//--------------------------------------------------------------------//
/**
 * ʂ̐ݒ
 */
  protected void setPaint(){
    super.setPaint();
    bg.setFont(font);
    int x1=panelSize.width/10;
    int x2=3*panelSize.width/5;
    int w=7*panelSize.width/20;
    int y1=panelSize.height/10;
    int y2=6*panelSize.height/10;
    int h=4*panelSize.height/10;
    pvWindow.setBounds(x1,y1,w,h);
    tsWindow.setBounds(x2,y1,w,h);
    pvWindow.setAxis();
    tsWindow.setAxis();
    infoBox.setLocation(x1,y2);
  }

//--------------------------------------------------------------------//
/**
 * Ot`悷
 * @param g `悷OtBbNXReLXg
 */
  protected void paintGraph(Graphics2D g){
    g.setColor(bgColor);
    g.fillRect(0,0,panelSize.width,panelSize.height);
    pvWindow.paint(g,fgColor,subColor);
    tsWindow.paint(g,fgColor,subColor);
    infoBox.paint(g);
  }

//--------------------------------------------------------------------//
/**
 * G[bZ[W\
 * @param g `悷OtBbNXReLXg
 * @param message G[bZ[W
 */
  protected void drawErrorMessage(Graphics2D g,String message){
    g.setColor(bgColor);
    g.fillRect(0,0,panelSize.width,panelSize.height);
    g.setColor(fgColor);
    super.drawErrorMessage(g,message);
  }
}
