package rankine.steamtable;
import java.awt.geom.Point2D;

//--------------------------------------------------------------------//
/**
 * OaCf[^
 */
public class SaturatedVapor{

/** f[^ */
  protected int count;

/** xi[z */
  protected double t[];

/** ͂i[z */
  protected double p[];

/** t̑p[^i[z */
  protected VaporVariable vvf[];

/** Cp[^i[z */
  protected VaporVariable vvg[];

/**
 * OaCf[^\z
 */
  public SaturatedVapor(){
    count=0;
    t=new double[count];
    p=new double[count];
    vvf=new VaporVariable[count];
    vvg=new VaporVariable[count];
  }

//--------------------------------------------------------------------//
/**
 * f[^Ԃ
 * @return f[^
 */
  public int getCount(){
    return count;
  }

//--------------------------------------------------------------------//
/**
 * f[^ݒ肷
 * @param count f[^
 */
  public void setCount(int count){
    this.count=count;
    t=new double[count];
    p=new double[count];
    vvf=new VaporVariable[count];
    vvg=new VaporVariable[count];
  }

//--------------------------------------------------------------------//
/**
 * f[^ݒ肷
 * @param index f[^̃CfbNX
 * @param t x
 * @param p 
 * @param vvf t̑p[^
 * @param vvg Cp[^
 */
  public void setParameter(int index,double t,double p,
      	      	      	   VaporVariable vvf,VaporVariable vvg){
    this.t[index]=t;
    this.p[index]=p;
    this.vvf[index]=vvf;
    this.vvg[index]=vvg;
  }

//--------------------------------------------------------------------//
/**
 * P-V }̖Oatf[^Ԃ
 * @return Oatf[^
 */
  public Point2D[] getLiquidPV(){
    Point2D pt[]=new Point2D[count];
    for(int i=0;i<count;i++){
      pt[i]=new Point2D.Double(vvf[i].v,p[i]);
    }
    return pt;
  }

//--------------------------------------------------------------------//
/**
 * T-S }̖Oatf[^Ԃ
 * @return Oatf[^
 */
  public Point2D[] getLiquidTS(){
    Point2D pt[]=new Point2D[count];
    for(int i=0;i<count;i++){
      pt[i]=new Point2D.Double(vvf[i].s,t[i]);
    }
    return pt;
  }

//--------------------------------------------------------------------//
/**
 * P-V }̖OaCf[^Ԃ
 * @return OaCf[^
 */
  public Point2D[] getGasPV(){
    Point2D pt[]=new Point2D[count];
    for(int i=0;i<count;i++){
      pt[count-1-i]=new Point2D.Double(vvg[i].v,p[i]);
    }
    return pt;
  }

//--------------------------------------------------------------------//
/**
 * T-S }̖OaCf[^Ԃ
 * @return OaCf[^
 */
  public Point2D[] getGasTS(){
    Point2D pt[]=new Point2D[count];
    for(int i=0;i<count;i++){
      pt[count-1-i]=new Point2D.Double(vvg[i].s,t[i]);
    }
    return pt;
  }

//--------------------------------------------------------------------//
/**
 * Od_̈͂Ԃ
 * @return Od_̈
 */
  public double getTripleP(){
    return p[0];
  }

//--------------------------------------------------------------------//
/**
 * Od_̉xԂ
 * @return Od_̉x
 */
  public double getTripleT(){
    return t[0];
  }

//--------------------------------------------------------------------//
/**
 * ՊE͂Ԃ
 * @return ՊE
 */
  public double getCriticalP(){
    return p[count-1];
  }

//--------------------------------------------------------------------//
/**
 * ՊExԂ
 * @return ՊEx
 */
  public double getCriticalT(){
    return t[count-1];
  }

//--------------------------------------------------------------------//
/**
 * ͂ɑΉOatf[^Ԃ
 * @param pp 
 * @return ԗ
 */
  public StateVariable getLiquidByP(double pp){
    if(pp<=p[0]){
      return new StateVariable(t[0],pp,vvf[0]);
    }
    else if(pp>=p[count-1]){
      return new StateVariable(t[count-1],pp,vvf[count-1]);
    }
    else{
      int i=PressureTable.serachIndex(0,count-1,pp,p);
      double cc=(pp-p[i])/(p[i+1]-p[i]);
      return new StateVariable(t[i]+cc*(t[i+1]-t[i]),pp,
      	      	      	       vvf[i].v+cc*(vvf[i+1].v-vvf[i].v),
      	      	      	       vvg[i].u+cc*(vvg[i+1].u-vvg[i].u),
      	      	      	       vvf[i].h+cc*(vvf[i+1].h-vvf[i].h),
      	      	      	       vvf[i].s+cc*(vvf[i+1].s-vvf[i].s));
    }
  }

//--------------------------------------------------------------------//
/**
 * ͂ɑΉOaCf[^Ԃ
 * @param pp 
 * @return ԗ
 */
  public StateVariable getGasByP(double pp){
    if(pp<=p[0]){
      return new StateVariable(t[0],pp,vvg[0]);
    }
    else if(pp>=p[count-1]){
      return new StateVariable(t[count-1],pp,vvg[count-1]);
    }
    else{
      int i=PressureTable.serachIndex(0,count-1,pp,p);
      double cc=(pp-p[i])/(p[i+1]-p[i]);
      return new StateVariable(t[i]+cc*(t[i+1]-t[i]),pp,
      	      	      	       vvg[i].v+cc*(vvg[i+1].v-vvg[i].v),
      	      	      	       vvg[i].u+cc*(vvg[i+1].u-vvg[i].u),
      	      	      	       vvg[i].h+cc*(vvg[i+1].h-vvg[i].h),
      	      	      	       vvg[i].s+cc*(vvg[i+1].s-vvg[i].s));
    }
  }

//--------------------------------------------------------------------//
/**
 * Ggs[ɑΉOaCf[^Ԃ
 * @param ss Ggs[
 * @return ԗ
 */
  public StateVariable getGasByS(double ss){
    if(ss>=vvg[0].s){
      return new StateVariable(t[0],p[0],vvg[0]);
    }
    else if(ss<=vvg[count-1].s){
      return new StateVariable(t[count-1],p[count-1],vvg[count-1]);
    }
    else{
      int i=PressureTable.serachIndex(0,count-1,ss,vvg);
      double cc=(ss-vvg[i].s)/(vvg[i+1].s-vvg[i].s);
      return new StateVariable(t[i]+cc*(t[i+1]-t[i]),
      	      	      	       p[i]+cc*(p[i+1]-p[i]),
      	      	      	       vvg[i].v+cc*(vvg[i+1].v-vvg[i].v),
      	      	      	       vvg[i].u+cc*(vvg[i+1].u-vvg[i].u),
      	      	      	       vvg[i].h+cc*(vvg[i+1].h-vvg[i].h),ss);
    }
  }
}
